@@ -3,11 +3,13 @@
 #!/usr/bin/python3
 
 import os,sys,time,re,traceback,platform
-from utilities.Qt import QtGui, QtCore, QtWidgets
+from PyQt5 import QtGui, QtCore, QtWidgets
+import KuttyPyLib
 
 from utilities.templates import ui_layout as layout
-from utilities import dio,REGISTERS,uploader
-import constants,inspect
+from utilities import dio,REGISTERS,uploader,syntax
+import constants
+import inspect
 
 
 from functools import partial
@@ -15,6 +17,19 @@ from collections import OrderedDict
 
 
 
+# translation stuff
+def translate(lang = None):
+	global app,t,t1
+	if lang is None:
+		lang=QtCore.QLocale.system().name()
+	t=QtCore.QTranslator()
+	t.load("lang/"+lang, os.path.dirname(__file__))
+	app.installTranslator(t)
+	t1=QtCore.QTranslator()
+	t1.load("qt_"+lang,
+		QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
+	app.installTranslator(t1)
+
 
 class myTimer():
 	def __init__(self,interval):
@@ -39,9 +54,12 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 	ports = ['A','B','C','D']
 	logThis = QtCore.pyqtSignal(str)
 	logThisPlain = QtCore.pyqtSignal(bytes)
-	serialGaugeSignal = QtCore.pyqtSignal(int)
+	serialGaugeSignal = QtCore.pyqtSignal(bytes)
+	serialGaugeConvert = 'bytes'
+	serialStream = b''
 	def __init__(self, parent=None,**kwargs):
 		super(AppWindow, self).__init__(parent)
+
 		self.setupUi(self)
 
 		self.VERSION = REGISTERS.VERSION_ATMEGA32 #This needs to be dynamically changed when hardware is connected
@@ -51,12 +69,15 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		
 		self.docks = [self.padock,self.pbdock,self.pcdock,self.pddock]
 		self.sensorList = []
+		self.controllerList = []
 		self.monitoring = True
 		self.logRegisters = True
 		self.userHexRunning = False
 		self.uploadingHex = False
 		self.autoUpdateUserRegisters = False
 		self.CFile = None #'~/kuttyPy.c'
+		self.ipy = None
+		self.defaultDirectory = path["examples"]
 
 		self.setTheme("material")
 		examples = [a for a in os.listdir(os.path.join(path["examples"],self.EXAMPLES_DIR)) if ('.py' in a) and a is not 'kuttyPy.py'] #.py files except the library
@@ -65,6 +86,14 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		if blinkindex!=-1: #default example. blink.py present in examples directory
 			self.exampleList.setCurrentIndex(blinkindex)
 
+		self.serialGuageButton.hide() #Hide the serial guage button
+		#Define some keyboard shortcuts for ease of use
+		self.shortcutActions={}
+		self.shortcuts={"f":partial(self.setLanguage,'fr_FR'),"e":partial(self.setLanguage,'en_IN'),"m":partial(self.setLanguage,'ml_IN')}
+		for a in self.shortcuts:
+			shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(a), self)
+			shortcut.activated.connect(self.shortcuts[a])
+			self.shortcutActions[a] = shortcut
 		######## PYTHON CODE
 		self.codeThread = QtCore.QThread()
 		self.codeEval = self.codeObject(self.REGISTERS)
@@ -89,6 +118,9 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 
 		self.uploadThread.started.connect(self.UploadObject.execute)
 		self.uploadThread.finished.connect(self.codeFinished)
+		
+		########## C CODE EDITOR SYNTAX HIGHLIGHTER
+		self.highlight = syntax.PythonHighlighter(self.editor.document())
 
 		self.commandQ = []
 		self.btns={}
@@ -102,14 +134,6 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 
 
 		self.initializeCommunications()
-		self.updatedRegs=OrderedDict()
-		self.currentRegister = 0
-		self.getRegs=[
-		('PINA',self.updateInputs),
-		('PINB',self.updateInputs),
-		('PINC',self.updateInputs),
-		('PIND',self.updateInputs),
-		]
 		self.pending = {
 		'status':myTimer(constants.STATUS_UPDATE_INTERVAL),
 		'update':myTimer(constants.AUTOUPDATE_INTERVAL),
@@ -118,7 +142,13 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		serialgaugeoptions = {'name':'Serial Monitor', 'init':print, 'read':None,
 				'fields':['Value'],
 				'min':[0],
-				'max':[255]}
+				'max':[1000],
+				'config':[{
+					'name':'Data Type',
+					'options':['byte','ASCII'],
+					'function':self.configSerialGauge
+					}
+			]}
 		self.serialGauge = dio.DIOSENSOR(self,serialgaugeoptions)
 		
 		self.startTime = time.time()
@@ -157,17 +187,33 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 				self.btns[name] = checkbox
 
 	def tabChanged(self,index):
+		if self.userHexRunning: #Firmware was running. Stop it.
+			self.launchFirmwareButton.setChecked(False)			
+			self.jumpToApplication(False)	
+
 		if index != 0 : #examples/editor tab. disable monitoring
 			self.monitoring = False
 			for a in self.docks:
 				a.hide()		
-		else: #Playground . enable monitoring and control.
+		else: #Playground . enable monitoring and control.				
 			self.monitoring = True
 			self.autoRefreshUserRegisters.setChecked(False)
 			self.userRegistersAutoRefresh(False)
+			self.setLogType('playground')
 			for a in self.docks:
 				a.show()		
 
+	def setLogType(self,tp):
+		self.log.clear()
+		if tp == 'playground':
+			self.logLabel.setText("Monitor registers being read and set during each operation")
+		elif tp == 'avr':
+			self.logLabel.setText("Avr-Gcc compile/upload messages")
+		elif tp == 'monitor':
+			self.logLabel.setText("Monitor data input from the serial port")
+		elif tp == 'python':
+			self.logLabel.setText("Python Code operations monitor")
+
 	################USER CODE SECTION####################
 	class codeObject(QtCore.QObject):
 		finished = QtCore.pyqtSignal()
@@ -248,15 +294,11 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			self.finished.emit()
 	
 	def runCode(self):
-		#if self.p:
-		#	try:
-		#		self.p.fd.read() #Clear junk
-		#		self.p.fd.close()
-		#	except:pass
 		if self.codeThread.isRunning():
 			print('one code is already running')
 			return
 
+		self.setLogType('python')
 		self.log.clear() #clear the log window
 		self.log.setText('''<span style="color:green;">----------User Code Started-----------</span>''')
 		kwargs = {}
@@ -285,7 +327,14 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			self.log.append('''<span style="color:red;">----------Kill Signal(Doesn't work yet. Restart the application)-----------</span>''')
 			self.codeThread.quit()
 			self.codeThread.terminate()
-		
+			del self.codeThread
+			self.codeThread = QtCore.QThread()
+			self.codeEval = self.codeObject(self.REGISTERS)
+			self.codeEval.moveToThread(self.codeThread)
+			self.codeEval.finished.connect(self.codeThread.quit)
+			self.codeEval.logThis.connect(self.appendLog) #Connect to the log window			
+			self.codeThread.started.connect(self.codeEval.execute)
+			self.codeThread.finished.connect(self.codeFinished)
 
 	def getReg(self,reg,record = True):
 		val = self.p.getReg(reg)
@@ -306,8 +355,26 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		self.log.moveCursor(QtGui.QTextCursor.End)
 		self.log.insertPlainText(txt.decode('ascii'))
 
-	def setSerialgauge(self,val):
-		self.serialGauge.setValue([val])
+	def configSerialGauge(self,val):
+		if val == 0: #'byte' 
+			print('Byte Mode(0-255)')
+			for a in self.serialGauge.gauges:
+				a.set_MaxValue(255)
+			self.serialGaugeConvert = 'bytes'
+		if val == 1: #'ascii' 
+			print('ASCII(0-10000)')
+			for a in self.serialGauge.gauges:
+				a.set_MaxValue(10000)
+			self.serialGaugeConvert = 'ascii'
+
+	def setSerialgauge(self,vals):
+		if self.serialGaugeConvert == 'bytes':
+			self.serialGauge.setValue([vals[0]])
+		elif self.serialGaugeConvert == 'ascii':
+			self.serialStream += vals
+			while b'\n' in self.serialStream:
+				val,_,self.serialStream = self.serialStream.partition(b'\n')
+				self.serialGauge.setValue([float(val)])
 
 	def genLog(self):
 		html='''<table border="1" align="center" cellpadding="1" cellspacing="0" style="font-family:arial,helvetica,sans-serif;font-size:9pt;">
@@ -342,7 +409,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		if self.userHexRunning:
 			t = self.p.fd.read(self.p.fd.in_waiting)
 			if len(t):
-				self.serialGaugeSignal.emit(t[0])
+				self.serialGaugeSignal.emit(t)
 				self.logThisPlain.emit(t)
 			return
 		
@@ -355,7 +422,6 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			for a in range(self.registerList.count()):
 				self.registerList.itemWidget(self.registerList.item(a)).execute()
 			
-
 		while len(self.commandQ):
 			if not self.centralWidget().isEnabled():
 				return
@@ -366,7 +432,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 				reg = self.getReg(pname)
 				reg &=~ (1<<bit)
 				if(a[2]):reg |= (1<<bit)
-				self.setReg(pname,reg);
+				self.setReg(pname,reg)
 			elif a[0] == 'DTYPE': #Digital pin I/O ['DTYPE','Pxx',state]
 				pname = 'DDR'+a[1][1].upper()
 				bit = int(a[1][2])
@@ -426,6 +492,13 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			elif type(btn)==dio.DIOCNTR and btn.currentPage==2: # CNTR
 					self.commandQ.append(['CNTR1',btn.slider])
 
+	def newStepperController(self):
+		if self.p.connected:
+			dialog = dio.DIOSTEPPER(self,total=200,device=self.p)
+			dialog.launch()
+			self.sensorList.append([dialog,None])
+
+
 	############ I2C SENSORS #################
 	def I2CScan(self):
 		if self.p.connected:
@@ -435,6 +508,10 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 				a[0].setParent(None)
 				a[1].setParent(None)
 			self.sensorList = []
+			for a in self.controllerList:
+				a[0].setParent(None)
+				a[1].setParent(None)
+			self.controllerList = []
 			for a in x:
 				s = self.p.sensors.get(a,None)
 				if s is not None:
@@ -451,7 +528,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 					dialog = dio.DIOCONTROL(self,s)
 					btn.clicked.connect(dialog.launch)
 					self.sensorLayout.addWidget(btn)
-					#self.sensorList.append([dialog,btn])
+					self.controllerList.append([dialog,btn])
 					continue
 
 				s = self.p.special.get(a,None)
@@ -460,7 +537,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 					dialog = dio.DIOROBOT(self,s)
 					btn.clicked.connect(dialog.launch)
 					self.sensorLayout.addWidget(btn)
-					#self.sensorList.append([dialog,btn])
+					self.controllerList.append([dialog,btn])
 					continue
 
 
@@ -484,45 +561,62 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			self.mode = mode
 
 		def execute(self):
-			if self.mode == 'compileupload':
+			if 'compile' in self.mode:
 				try:
 					import subprocess
-					fname = self.fname.split(".")[0]
-					cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o %s %s' %('atmega32',fname,self.fname)
+					if self.fname[-2:] in ['.s','.S']:
+						action = 'Assembl'
+					else:#elif self.fname[-2:] in ['.c','.C']:
+						action = 'Compil'
+					fname = '.'.join(self.fname.split('.')[:-1])
+					if self.p.version == REGISTERS.VERSION_ATMEGA32 or self.p.connected==False: #by default, compile for ATMEGA32.
+						cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o "%s" "%s"' %('atmega32',fname,self.fname)
+						self.logThis.emit('''<span style="color:green;">%sing for Atmega32</span>'''%(action))
+					elif self.p.version == REGISTERS.VERSION_ATMEGA328P:
+						cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o "%s" "%s"' %('atmega328p',fname,self.fname)
+						self.logThis.emit('''<span style="color:green;">%sing for Atmega328p (Nano)</span>'''%(action))
+					else:
+						self.logThis.emit('''<span style="color:red;">%ser UNAVAILABLE</span>'''%(action))
+						return
+					print(cmd)
 					res = subprocess.getstatusoutput(cmd)
 					if res[0] != 0:
-						self.logThis.emit('''<span style="color:red;">Compile Error: %s</span>'''%res[1])
+						self.logThis.emit('''<span style="color:darkred;">%se Error: %s</span>'''%(action,res[1].replace('\n','<br>')) )
 						self.finished.emit()
 						return
 
 					else:
-						self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
-					cmd = 'avr-objcopy -j .text -j .data -O ihex %s %s.hex' %(fname,fname)
+						self.logThis.emit('''<span style="color:gray;">%s</span><br>'''%res[1])
+					cmd = 'avr-objcopy -j .text -j .data -O ihex "%s" "%s.hex"' %(fname,fname)
 					res = subprocess.getstatusoutput(cmd)
-					self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
-					cmd = 'avr-objdump -S %s > %s.lst'%(fname,fname)
+					self.logThis.emit(res[1])#'''<span style="color:gray;">%s</span><br>'''%res[1])
+					cmd = 'avr-objdump -S "%s" > "%s.lst"'%(fname,fname)
 					res = subprocess.getstatusoutput(cmd)
-					self.logThis.emit('''<span style="color:white;">%s</span><br>'''%res[1])
-					if self.fname[-2:] in ['.c','.C']:
-						self.fname = self.fname[:-2]+'.hex' #Replace .c with .hex
-						self.mode = 'upload'
-						self.logThis.emit('''<span style="color:green;">Generated Hex File</span>''')
-					self.logThis.emit('''<span style="color:green;">Finished Compiling: Generated Hex File</span>''')
+					self.logThis.emit(res[1])#'''<span style="color:gray;">%s</span><br>'''%res[1])
+					self.logThis.emit('Finished %sing: Generated Hex File'%action) #'''<span style="color:darkgreen;">Finished %sing: Generated Hex File</span>'''%(action))
 				except Exception as err:
-					self.logThis.emit('''<span style="color:red;">Failed to Compile:%s</span>'''%str(err))
+					self.logThis.emit('''<span style="color:darkred;">Failed to %se:%s</span>'''%str(action,err))
 
 			if self.p.connected:
-				if self.mode == 'upload':
+				if 'upload' in self.mode:
 					try:
-						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.4)
+						if self.fname[-2:] in ['.c','.C','.s','.S']:
+							self.fname = self.fname[:-2]+'.hex' #Replace .c with .hex
+						self.logThis.emit('''<span style="font-size:12pt">Upload Code... Trigger Reset...</span>''')
+						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.1)
+						self.p.get_version()
 						dude = uploader.Uploader(self.p.fd, hexfile=self.fname,logger = self.logThis)
 						dude.program()
 						dude.verify()
 						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.2)
 						self.p.get_version()
-						self.logThis.emit('''<span style="color:green;">Finished upload</span>''')
+						self.logThis.emit('''<span style="color:darkgreen;">Finished upload</span>''')
 					except Exception as err:
-						self.logThis.emit('''<span style="color:red;">Failed to upload</span>''')
+						print('upload error',err)
+						self.p.fd.setRTS(0);time.sleep(0.01);self.p.fd.setRTS(1);time.sleep(0.2)
+						self.p.get_version()
+						self.logThis.emit('''<span style="color:darkred;">Failed to upload</span>''')
+						#self.jumpToApplication(False) #Force a reset
 			self.finished.emit()
 	
 	def uploadHex(self):
@@ -538,25 +632,53 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 
 
 	def openFile(self):
-		filename = QtWidgets.QFileDialog.getOpenFileName(self," Open a C file to edit", path["examples"], "C Files (*.c *.C)")
+		filename = QtWidgets.QFileDialog.getOpenFileName(self," Open a C/Asm file to edit", self.defaultDirectory, "C/ASM Files (*.c *.C *.s *.S);; Asm Files (*.s *.S);; C Files (*.c *.C)")
 		if len(filename[0]):
+			self.defaultDirectory = ''
 			self.filenameLabel.setText(filename[0])
 			self.CFile = filename[0]
 			self.log.clear()
+			infile = open(filename[0], 'r')
+			self.editor.setPlainText(infile.read())
 			self.log.setText('''<span style="color:cyan;">-- Opened File: %s --</span><br>'''%filename[0])
-			if 'inux' in platform.system(): #Linux based system
-				os.system('%s %s' % ('xdg-open', filename[0]))
-			else:
-				os.system('%s %s' % ('open', filename[0]))
+
 
 	def compileAndUpload(self):
+		self.setLogType('avr')
+		self.saveFile()
 		if self.CFile:
 			self.uploadingHex = True
-			self.log.clear()
 			self.log.setText('''<span style="color:cyan;">-- Compiling and Uploading Code --</span><br>''')
 			self.UploadObject.config('compileupload',self.p,self.CFile)
 			self.uploadThread.start()
+
+	def compile(self):
+		self.setLogType('avr')
+		self.saveFile()
+		if self.CFile:
+			self.log.setText('''<span style="color:cyan;">-- Compiling Code --</span><br>''')
+			self.UploadObject.config('compile',self.p,self.CFile)
+			self.uploadThread.start()
+
+	def upload(self):
+		self.setLogType('avr')
+		if self.CFile:
+			self.uploadingHex = True
+			self.log.setText('''<span style="color:cyan;">-- Uploading Code --</span><br>''')
+			self.UploadObject.config('upload',self.p,self.CFile)
+			self.uploadThread.start()
+
 			
+	def saveFile(self):
+		if not self.CFile:
+			name = QtGui.QFileDialog.getSaveFileName(self, 'Save File')
+			print('created new file:',name)
+			self.CFile = name[0]
+			self.log.setText('''<span style="color:green;">-- Created new file: %s --</span><br>'''%self.CFile)
+		fn = open(self.CFile,'w')
+		fn.write(self.editor.toPlainText())
+		fn.close()
+		self.log.setText('''<span style="color:green;">-- Saved to: %s --</span><br>'''%self.CFile)
 
 	##############################
 	def setTheme(self,theme):
@@ -573,25 +695,55 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 		else:
 			self.p = KuttyPyLib.connect(autoscan=True)
 		if self.p.connected:
-			self.userApplication.setChecked(False)
+			self.launchFirmwareButton.setChecked(False)
 			self.setWindowTitle('KuttyPy Interactive Console [{0:s}]'.format(self.p.portname))
+			self.updatedRegs=OrderedDict()
+			self.currentRegister = 0
+			self.VERSION = self.p.version
+			self.SPECIALS = REGISTERS.VERSIONS[self.VERSION]['SPECIALS']
+			self.REGISTERS = REGISTERS.VERSIONS[self.VERSION]['REGISTERS']
+			self.EXAMPLES_DIR = REGISTERS.VERSIONS[self.VERSION]['examples directory']
+			if self.p.version == REGISTERS.VERSION_ATMEGA328P:
+				self.getRegs=[
+				('PINB',self.updateInputs),
+				('PINC',self.updateInputs),
+				('PIND',self.updateInputs),
+				]
+			elif self.p.version == REGISTERS.VERSION_ATMEGA32:
+				self.getRegs=[
+				('PINA',self.updateInputs),
+				('PINB',self.updateInputs),
+				('PINC',self.updateInputs),
+				('PIND',self.updateInputs),
+				]
+
+
 		else:
 			self.setWindowTitle('KuttyPy Interactive Console [ Hardware not detected ]')
 
+	def setLanguage(self,lang = 'fr_FR'):
+		translate(lang)
+		self.retranslateUi(self)
+
+	def showSerialGauge(self):
+		self.serialGauge.show()
+
 	def jumpToApplication(self,state):
 		if self.p:
 			if state:
+				self.serialGuageButton.show()
 				self.userHexRunning=True
 				self.p.fd.write(b'j') #Skip to application (Bootloader resets) 
 
 				for a in self.docks:
 					a.setEnabled(False)
-				self.tabs.setEnabled(False)
-				self.log.clear()
+				#self.tabs.setEnabled(False)
+				self.setLogType('monitor')
 				self.log.setText('''<span style="color:cyan;">-- Serial Port Monitor --</span><br>''')
-				self.serialGauge.show()
+				#self.serialGauge.show()
 
 			else:
+				self.serialGuageButton.hide()
 				self.p.fd.setRTS(0)  #Trigger a reset
 				time.sleep(0.01)
 				self.p.fd.setRTS(1)
@@ -602,38 +754,46 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 				for a in self.docks:
 					a.setEnabled(True)
 				self.userHexRunning=False
-				self.tabs.setEnabled(True)
+				#self.tabs.setEnabled(True)
 				self.serialGauge.hide()
 		else:
 			if self.isChecked():
 				self.setChecked(False)
 
+		
 	def makeBottomMenu(self):
 		try:self.pushbutton.setParent(None)
 		except:pass
 		self.pushbutton = QtWidgets.QPushButton('Menu')
-		self.pushbutton.setStyleSheet("height: 13px;padding:3px;color: #FFFFFF;")
+		self.pushbutton.setStyleSheet("height: 13px;padding:3px;color: #7c7;")
 		menu = QtWidgets.QMenu()
 
 		menu.addAction('Save Window as Svg', self.exportSvg)
+		menu.addAction('Open Stepper Controller', self.newStepperController)
 
 		#Theme
+		'''
 		self.themeAction = QtWidgets.QWidgetAction(menu)
 		themes = [a.split('.qss')[0] for a in os.listdir(path["themes"]) if '.qss' in a]
 		self.themeBox = QtWidgets.QComboBox(); self.themeBox.addItems(themes)
 		self.themeBox.currentIndexChanged['QString'].connect(self.setTheme)
 		self.themeAction.setDefaultWidget(self.themeBox)
 		menu.addAction(self.themeAction)
+		'''
+		themes = [a.split('.qss')[0] for a in os.listdir(path["themes"]) if '.qss' in a]
+		self.themeBox = QtWidgets.QComboBox(); self.themeBox.addItems(themes)
+		self.themeBox.currentIndexChanged['QString'].connect(self.setTheme)
+		self.statusBar.addPermanentWidget(self.themeBox)
 
 		self.pushbutton.setMenu(menu)
 
-		self.userApplication = QtWidgets.QCheckBox("User App");
-		self.userApplication.toggled['bool'].connect(self.jumpToApplication)
-		self.statusBar.addPermanentWidget(self.userApplication)
+		#self.userApplication = QtWidgets.QCheckBox("User App");
+		#self.userApplication.toggled['bool'].connect(self.jumpToApplication)
+		#self.statusBar.addPermanentWidget(self.userApplication)
 
-		self.hexUploadButton = QtWidgets.QPushButton("Upload Hex");
-		self.hexUploadButton.clicked.connect(self.uploadHex)
-		self.statusBar.addPermanentWidget(self.hexUploadButton)
+		#self.hexUploadButton = QtWidgets.QPushButton("Upload Hex");
+		#self.hexUploadButton.clicked.connect(self.uploadHex)
+		#self.statusBar.addPermanentWidget(self.hexUploadButton)
 
 
 		self.speedbutton = QtWidgets.QComboBox(); self.speedbutton.addItems(['Slow','Fast','Ultra']);
@@ -658,8 +818,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 						self.setWindowTitle('Error : Device Disconnected')
 						QtWidgets.QMessageBox.warning(self, 'Connection Error', 'Device Disconnected. Please check the connections')
 						try:
-							self.p.fd.close()
-							self.p.portname = None
+							self.p.close()
 						except:pass
 						self.p.connected = False
 						self.setWindowTitle('KuttyPy Interactive Console [ Hardware not detected ]')
@@ -693,7 +852,7 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 
 	######## WINDOW EXPORT SVG
 	def exportSvg(self):
-		from utilities.Qt import QtSvg
+		from PyQt5 import QtSvg
 		path, _filter  = QtWidgets.QFileDialog.getSaveFileName(self, 'Save File', '~/')
 		if path:
 			generator = QtSvg.QSvgGenerator()
@@ -708,8 +867,14 @@ class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
 			self.render(p)
 			p.end()
 
-
-
+	def ipython(self): #Experimental feature. Import ipython and launch console
+		if not self.p.connected:
+			return
+		from utilities import ipy
+		if not self.ipy:
+			self.ipy = ipy.AppWindow(self,kp = self.p)
+		self.ipy.show()
+		self.ipy.updateDevice(self.p)
 
 def translators(langDir, lang=None):
 	"""
@@ -780,13 +945,11 @@ def common_paths():
 	return path
 
 
-
-
-if __name__ == "__main__":
+def run():
+	global path, app, myapp
 	path = common_paths()
+	print('QT Version',QtWidgets.__file__)
 	app = QtWidgets.QApplication(sys.argv)
-	import KuttyPyLib
-
 	myapp = AppWindow(app=app, path=path)
 	myapp.show()
 	r = app.exec_()
@@ -800,5 +963,7 @@ if __name__ == "__main__":
 	app.deleteLater()
 	sys.exit(r)
 
+if __name__ == "__main__":
+	run()
 
 
