@@ -1,30 +1,32 @@
 #include "mh-utils.c"
 
-#define DELAY 5
+#define DELAY 10
+#define STEPS 40
 int main (void)
   {
 DDRB = 15;  //For controlling the stepper motor
+uint8_t steps[]={0b1100,0b0110,0b0011,0b1001};
 uint16_t pos = 0;
   for(;;)
 	{
-	for(pos=0;pos<80;pos++){
-		PORTB=3;
+	for(pos=0;pos<STEPS;pos++){
+		PORTB=steps[2];
 		delay_ms(DELAY);
-		PORTB=6;
+		PORTB=steps[1];
 		delay_ms(DELAY);
-		PORTB=12;
+		PORTB=steps[0];
 		delay_ms(DELAY);
-		PORTB=9;
+		PORTB=steps[3];
 		delay_ms(DELAY);
 		}
-	for(pos=0;pos<80;pos++){
-		PORTB=12;
+	for(pos=0;pos<STEPS;pos++){
+		PORTB=steps[0];
 		delay_ms(DELAY);
-		PORTB=6;
+		PORTB=steps[1];
 		delay_ms(DELAY);
-		PORTB=3;
+		PORTB=steps[2];
 		delay_ms(DELAY);
-		PORTB=9;
+		PORTB=steps[3];
 		delay_ms(DELAY);
 		}
   }
