@@ -1,10 +1,6 @@
 #include <avr/io.h>
 
-void delay_ms (uint16_t k)  // idle for k milliseconds, only for 8MHz clock
-    {
-    volatile uint16_t x;
-    while(k--) {x=532; while (x--);}
-    }
+extern void delay_ms(uint16_t);
 
 void tc1_demo()    // this should be the zero crossing ISR
 {
@@ -12,7 +8,8 @@ TCNT1 = 0;
 // OC1RA should be set according to the ADC output
 }
 
-main(){
+int main(){
+  delay_ms(10);
   TCCR1B = (1 << WGM12) | (1 <<CS11);	// Normal mode, 1 usec with 8MHz Clock
   TCCR1A = (1 << COM1A1) |(1 << COM1A0);    // Set OC1A on match
   OCR1A = 4000;     // Set OC1A (PD5) after 4 milliseconds
@@ -23,4 +20,5 @@ main(){
   delay_ms(50);
   }
   
+return 1;
 }
